#include <iostream>
#include <memory>

using namespace std;

///////////////////////////////

//@@	Our dummy test class
//

class	Object
{
public:

	Object(const char *s)	{	cout << s << endl;	}

	~Object()				{	cout << "Object destructing" << endl;	}

	//@@	Our erroronus method

	void	Some_Method()
	{
		if ( 1 != 0 )			// fake some error
			throw 100;
	}
};

///////////////

//@@	SEH example

void	SEH_Function()
{
	Object *p = NULL;

	__try
	{
		// we cannot compile this because Object is a class (has destructor)
		// if Object was a struct, then we can use SEH exception handling

		//	p = new Object("SEH Function");
	}
	__finally
	{
		delete p;
	}
}

///////////////

//@@	Ugly method of preventing leaks

void	Ugly_Function()
{
	Object *p	= NULL;

	try
	{
		p	= new Object("Object from Ugly Function");

		p->Some_Method();		// uncomment to not cause an exception
	}
	catch(...)
	{
		delete	p;

		throw;		// rethrow the exception
	}

}

////////////////

//@@	Ultimate Smart Pointer solution

void	Smart_Pointer_Function()
{
	auto_ptr<Object>	p( new Object("Object from Smart Pointer Function") );

	p->Some_Method();	// uncomment to not cause an exception
}

////////////////

int	main()
{
	try
	{
		// uncomment to choose one method

		Ugly_Function();
		//	Smart_Pointer_Function();


		cout << "*** No exception encountered ***" << endl;		// should never reach here
	}
	catch( int i )
	{
		cout << "Caught an int exception of value " << i << endl;
	}
	catch(...)
	{
		cout << "Unknown exception caught" << endl;
	}

	return 1;
}

////////////////////////////////